@@ -1,12 +1,11 @@
 ;;; spacious-padding.el --- Increase the padding/spacing of frames and windows -*- lexical-binding: t -*-
 
-;; Copyright (C) 2023  Free Software Foundation, Inc.
+;; Copyright (C) 2023-2024  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
-;; Maintainer: Protesilaos Stavrou General Issues <~protesilaos/general-issues@lists.sr.ht>
-;; URL: https://git.sr.ht/~protesilaos/spacious-padding
-;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 0.3.0
+;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
+;; URL: https://github.com/protesilaos/spacious-padding
+;; Version: 0.4.0
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: convenience, focus, writing, presentation
 
@@ -60,7 +59,8 @@
      :mode-line-width 6
      :tab-width 4
      :right-divider-width 30
-     :scroll-bar-width 8)
+     :scroll-bar-width 8
+     :fringe-width 8)
   "Set the pixel width of individual User Interface elements.
 This is a plist of the form (:key1 value1 :key2 value2).  The
 value is always a natural number.  Keys are keywords among the
@@ -71,23 +71,46 @@ following:
   start.
 
 - `:right-divider-width' is the space between two side-by-side
-  windows.
+  windows.  If the value is 1, the border is not hidden when
+  `spacious-padding-mode' is enabled.
 
-- `:tab-width' concerns the padding around tab buttons.  For the
-  time being, `tab-bar-mode' is the only one affected.
+- `:fringe-width' applies to the fringes on either side of the
+  window.  The more specific keys `:left-fringe-width' and
+  `:right-fringe-wdith' can be used for finer control.  If those
+  are not specified (or set to a nil value), they fall back to
+  `:fringe-width'.  (The fringes are the window sides where line
+  wrapping and other indicators are displayed).
+
+- `:tab-width' concerns the padding around buttons of all tabbed
+  interfaces (`tab-bar-mode', `tab-line-mode').
+
+- `:tab-bar-width' concerns the padding around buttons of the
+  `tab-bar-mode'.  If not specified (or set to nil) it uses the
+  value of `:tab-width'.
+
+- `:tab-line-width' concerns the padding around buttons of the
+  tab-line.  If not specified (or set to nil) it uses the value
+  of `:tab-width'.
 
 - `:header-line-width', `mode-line-width', `scroll-bar-width'
   point to the header-line, mode-line, and scroll-bar,
-  respectively."
+  respectively.
+
+For the technicalities, read Info node `(elisp) Frame Layout'."
   :type '(plist
           :key-type (choice (const :internal-border-width)
                             (const :right-divider-width)
+                            (const :fringe-width)
+                            (const :left-fringe-width)
+                            (const :right-fringe-width)
                             (const :tab-width)
+                            (const :tab-bar-width)
+                            (const :tab-line-width)
                             (const :header-line-width)
                             (const :mode-line-width)
                             (const :scroll-bar-width))
-          :value-type natnum)
-  :package-version '(spacious-padding . "0.2.0")
+          :value-type (choice natnum (const nil)))
+  :package-version '(spacious-padding . "0.4.0")
   :group 'spacious-padding)
 
 (defcustom spacious-padding-subtle-mode-line nil
@@ -166,15 +189,22 @@ Examples of valid configurations:
   '(header-line header-line-highlight)
   "Header line faces relevant to `spacious-padding-mode'.")
 
-;; TODO 2023-11-16: Cover `tab-line-mode'.
-(defvar spacious-padding--tab-faces
+(defvar spacious-padding--tab-bar-faces
   '(tab-bar tab-bar-tab tab-bar-tab-inactive)
   "Tab faces relevant to `spacious-padding-mode'.")
 
-(defun spacious-padding--get-box-width (key)
+(defvar spacious-padding--tab-line-faces
+  '(tab-line tab-line-tab tab-line-tab-inactive)
+  "Tab faces relevant to `spacious-padding-mode'.")
+
+(defun spacious-padding--get-box-width (key &optional no-fallback)
   "Get width for :box of face represented by KEY in `spacious-padding-widths'.
-Return 4 if KEY does not have a value."
-  (or (plist-get spacious-padding-widths key) 4))
+Return 4 if KEY does not have a value.  If optional NO-FALLBACK
+is non-nil, do not return a fallback value: just nil."
+  (cond
+   ((plist-get spacious-padding-widths key))
+   (no-fallback nil)
+   (t 4)))
 
 (defun spacious-padding--get-face-width (face)
   "Return width of FACE from `spacious-padding-widths'."
@@ -183,8 +213,12 @@ Return 4 if KEY does not have a value."
     (spacious-padding--get-box-width :mode-line-width))
    ((memq face spacious-padding--header-line-faces)
     (spacious-padding--get-box-width :header-line-width))
-   ((memq face spacious-padding--tab-faces)
-    (spacious-padding--get-box-width :tab-width))
+   ((memq face spacious-padding--tab-bar-faces)
+    (or (spacious-padding--get-box-width :tab-bar-width :fall-back-to-tab-width)
+        (spacious-padding--get-box-width :tab-width)))
+   ((memq face spacious-padding--tab-line-faces)
+    (or (spacious-padding--get-box-width :tab-line-width :fall-back-to-tab-width)
+        (spacious-padding--get-box-width :tab-width)))
    (t (error "`%s' is not relevant to `spacious-padding-mode'" face))))
 
 (defun spacious-padding--get-face-overline-color (face fallback subtle-key)
@@ -217,6 +251,13 @@ overline."
           :color ,bg
           :style nil)))))
 
+(defun spacious-padding-set-window-divider (face color)
+  "Set window divider FACE to COLOR its width is greater than 1."
+  (list
+   face
+   `((t ,(when (> (plist-get spacious-padding-widths :right-divider-width) 1)
+           (list :background color :foreground color))))))
+
 (defun spacious-padding-set-invisible-dividers (_theme)
   "Make window dividers for THEME invisible."
   (let ((bg-main (face-background 'default))
@@ -234,9 +275,13 @@ overline."
      `(mode-line-highlight ((t :box (:color ,fg-main))))
      `(tab-bar-tab ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab 'tab-bar))))
      `(tab-bar-tab-inactive ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab-inactive 'tab-bar))))
-     `(window-divider ((t :background ,bg-main :foreground ,bg-main)))
-     `(window-divider-first-pixel ((t :background ,bg-main :foreground ,bg-main)))
-     `(window-divider-last-pixel ((t :background ,bg-main :foreground ,bg-main))))))
+     `(tab-line-tab ((t ,@(spacious-padding-set-face-box-padding 'tab-line-tab 'tab-line))))
+     `(tab-line-tab-inactive ((t ,@(spacious-padding-set-face-box-padding 'tab-line-tab-inactive 'tab-line))))
+     `(tab-line-tab-active ((t ,@(spacious-padding-set-face-box-padding 'tab-line-tab-active 'tab-line))))
+     `(vertical-border ((t :background ,bg-main :foreground ,bg-main)))
+     `(,@(spacious-padding-set-window-divider 'window-divider bg-main))
+     `(,@(spacious-padding-set-window-divider 'window-divider-first-pixel bg-main))
+     `(,@(spacious-padding-set-window-divider 'window-divider-last-pixel bg-main)))))
 
 (defun spacious-padding-unset-invisible-dividers ()
   "Make window dividers for THEME invisible."
@@ -252,6 +297,9 @@ overline."
    '(mode-line-highlight (( )))
    '(tab-bar-tab (( )))
    '(tab-bar-tab-inactive (( )))
+   '(tab-line-tab (( )))
+   '(tab-line-tab-inactive (( )))
+   `(vertical-border (( )))
    '(window-divider (( )))
    '(window-divider-first-pixel (( )))
    '(window-divider-last-pixel (( )))))
@@ -262,6 +310,15 @@ overline."
 (defvar spacious-padding--right-divider-width nil
   "Default value of frame parameter `right-divider-width'.")
 
+(defvar spacious-padding--fringe-width nil
+  "Default value of frame parameters `left-fringe' and `right-fringe'.")
+
+(defvar spacious-padding--left-fringe-width nil
+  "Default value of frame parameter `left-fringe'.")
+
+(defvar spacious-padding--right-fringe-width nil
+  "Default value of frame parameter `right-fringe'.")
+
 (defvar spacious-padding--scroll-bar-width nil
   "Default value of frame parameter `scroll-bar-width'.")
 
@@ -273,6 +330,14 @@ overline."
   (unless spacious-padding--right-divider-width
     (setq spacious-padding--right-divider-width
           (frame-parameter nil 'right-divider-width)))
+  (unless spacious-padding--fringe-width
+    (setq spacious-padding--fringe-width 8)) ; 8 is the default per `fringe-mode'
+  (unless spacious-padding--left-fringe-width
+    (setq spacious-padding--left-fringe-width
+          (frame-parameter nil 'left-fringe-width)))
+  (unless spacious-padding--right-fringe-width
+    (setq spacious-padding--right-fringe-width
+          (frame-parameter nil 'right-fringe-width)))
   (unless spacious-padding--scroll-bar-width
     (setq spacious-padding--scroll-bar-width
           (frame-parameter nil 'scroll-bar-width))))
@@ -292,6 +357,9 @@ parameter value."
 
 (spacious-padding--define-get-frame-param "internal-border-width" 0)
 (spacious-padding--define-get-frame-param "right-divider-width" 1)
+(spacious-padding--define-get-frame-param "fringe-width" 8)
+(spacious-padding--define-get-frame-param "left-fringe-width" nil)
+(spacious-padding--define-get-frame-param "right-fringe-width" nil)
 (spacious-padding--define-get-frame-param "scroll-bar-width" 16)
 
 (defun spacious-padding-modify-frame-parameters (reset)
@@ -301,6 +369,10 @@ parameter values."
   (modify-all-frames-parameters
    `((internal-border-width . ,(spacious-padding--get-internal-border-width reset))
      (right-divider-width . ,(spacious-padding--get-right-divider-width reset))
+     (left-fringe . ,(or (spacious-padding--get-left-fringe-width reset)
+                         (spacious-padding--get-fringe-width reset)))
+     (right-fringe . ,(or (spacious-padding--get-right-fringe-width reset)
+                          (spacious-padding--get-fringe-width reset)))
      (scroll-bar-width  . ,(spacious-padding--get-scroll-bar-width reset)))))
 
 (defun spacious-padding--enable-mode ()
